using System;
using System.Collections.Generic;
using System.Text;
using System.CodeDom.Compiler;
using Microsoft.VisualStudio.Modeling;
using System.CodeDom;
using System.Globalization;
using System.IO;
using System.Collections;
using Microsoft.VisualStudio.TextTemplating;
using Microsoft.VisualStudio.TextTemplating.VSHost;
using Microsoft.VisualStudio.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling;
using Microsoft.VisualStudio.EnterpriseTools.Application;
using Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.LogicalInfrastructureDesign.Modeling;
using Microsoft.VisualStudio.EnterpriseTools.LogicalInfrastructureDesign.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.ArtifactModel.Project;
using Microsoft.VisualStudio.EnterpriseTools.ArtifactModel.Asmx;
using Microsoft.VisualStudio.EnterpriseTools.ArtifactModel.Clr;
using Microsoft.VisualStudio.EnterpriseTools.Application.Modeling;
using Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.PlugIns;
using Microsoft.Win32;

namespace Eastbanc.VSIP.Sdm
{
    [ProvideDirectiveProcessor(typeof(ApplicationDirectiveProcessor), ApplicationDirectiveProcessor.DIRECTIVE_PROCESSOR_NAME, "A directive processor that provides access to System Definition Model")]
    class ApplicationDirectiveProcessor : RequiresProvidesDirectiveProcessor
    {
        public const String DIRECTIVE_PROCESSOR_NAME = "ApplicationModelDirectiveProcessor";
        public const String DIRECTIVE_NAME = "sdmmodel";

        public ApplicationDirectiveProcessor():base()
        {
            
        }

        public override bool IsDirectiveSupported(string directiveName)
        {
            return (StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0);
        }

        protected override string FriendlyName
        {
            get { return DIRECTIVE_PROCESSOR_NAME; }
        }

        protected override void InitializeRequiresDictionary(string directiveName, IDictionary<string, string> requiresDictionary)
        {
            if ((StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0))
            {
                requiresDictionary["fileName"] = null;
                requiresDictionary["folderName"] = null;
                requiresDictionary["filter"] = null;
                requiresDictionary["rootSystem"] = null;
            }
        }

        protected override void InitializeProvidesDictionary(string directiveName, IDictionary<string, string> providesDictionary)
        {
            if ((StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0))
            {
                providesDictionary["SolutionModelRoot"] = "SolutionModelRoot";
            }
        }

        /// <summary>
        /// Generate the code to access the model. Use to CodeDomProvider so we are language-agnostic
        /// </summary>
        /// <param name="directiveName"></param>
        /// <param name="codeBuffer"></param>
        /// <param name="languageProvider"></param>
        /// <param name="requiresArguments"></param>
        /// <param name="providesArguments"></param>
        protected override void GenerateTransformCode(string directiveName, StringBuilder codeBuffer, CodeDomProvider languageProvider, IDictionary<string, string> requiresArguments, IDictionary<string, string> providesArguments)
        {
            if ((StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0))
            {
                if (languageProvider != null)
                {
                    // Create a field to store the model once it is loaded
                    CodeMemberField modelroot = new CodeMemberField();
                    modelroot.Name = providesArguments["SolutionModelRoot"].ToLower(CultureInfo.InvariantCulture) + "Value";
                    modelroot.Type = new CodeTypeReference(typeof(Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling.SolutionModelRoot));
                    modelroot.Attributes = MemberAttributes.Private;

                    // Create a property for the Model that delay-loads the model
                    CodeMemberProperty modelrootProperty = new CodeMemberProperty();
                    modelrootProperty.Name = providesArguments["SolutionModelRoot"];
                    modelrootProperty.Type = new CodeTypeReference(typeof(Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling.SolutionModelRoot));
                    modelrootProperty.Attributes = MemberAttributes.Private;
                    modelrootProperty.HasSet = false;
                    modelrootProperty.HasGet = true;
                    modelrootProperty.GetStatements.Add(new CodeMethodReturnStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), modelroot.Name)));

                    // Generate the actual code using the CodeDomProvider
                    CodeGeneratorOptions options = new CodeGeneratorOptions();
                    options.BlankLinesBetweenMembers = true;
                    options.IndentString = "    ";
                    options.VerbatimOrder = true;
                    options.BracingStyle = "C";
                    using (StringWriter writer = new StringWriter(codeBuffer, CultureInfo.InvariantCulture))
                    {
                        languageProvider.GenerateCodeFromMember(modelroot, writer, options);
                        languageProvider.GenerateCodeFromMember(modelrootProperty, writer, options);
                    }
                }
            }
        }

        /// <summary>
        /// Contribute additively to initialization code for the TextTransformation generated class.
        /// </summary>
        /// <remarks>
        /// This code will be added before the call to the base class.
        /// </remarks>
        /// <param name="codeBuffer"></param>
        /// <param name="languageProvider"></param>
        /// <param name="requiresArguments"></param>
        /// <param name="providesArguments"></param>
        protected override void GeneratePreInitializationCode(string directiveName, StringBuilder codeBuffer, CodeDomProvider languageProvider, IDictionary<string, string> requiresArguments, IDictionary<string, string> providesArguments)
        {
            if ((StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0))
            {
                if (languageProvider != null)
                {

                    string[] domainModels = 
                    {
                typeof(CoreDesignSurface).FullName, 
                typeof(SdmCore).FullName, 
                typeof(SdmGeneric).FullName, 
                typeof(ApplicationModel).FullName, 
                typeof(PresentationSurface).FullName, 
                typeof(LidObjectModel).FullName, 
                typeof(LidPresentationModel).FullName, 
                typeof(SdmCoreSurface).FullName, 
                typeof(ProjectMetaModel).FullName, 
                typeof(AsmxMetaModel).FullName, 
                typeof(TypeModel).FullName,
                typeof(StandardPlugInsMetaModel).FullName
                    };
					

                    CodeGeneratorOptions options = new CodeGeneratorOptions();
                    options.BlankLinesBetweenMembers = true;
                    options.IndentString = "    ";
                    options.VerbatimOrder = true;
                    options.BracingStyle = "C";
                    using (StringWriter writer = new StringWriter(codeBuffer, CultureInfo.InvariantCulture))
                    {
                        foreach (string domainModel in domainModels)
                        {
                            CodeExpressionStatement addModel = new CodeExpressionStatement(new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeThisReferenceExpression(), "AddDomainModel"), new CodeTypeOfExpression(domainModel)));
                            languageProvider.GenerateCodeFromStatement(addModel, writer, options);
                        }
                    }
                }
            }
        }

        protected virtual void LoadInitializationModelFileNames(IList<string> fileExtentions)
        {
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\VisualStudio\8.0\EnterpriseTools\Sdm\InitializationFiles");

            if (key != null)
            {
                //foreach (string valName in key.GetValueNames())
                //{
                //    fileExtentions.Add(key.GetValue(valName) as String);
                //}
            }
        }

        /// <summary>
        /// Contribute additively to initialization code for the TextTransformation generated class.
        /// </summary>
        /// <remarks>
        /// This code will be added after the call to the base class.
        /// </remarks>
        /// <param name="codeBuffer"></param>
        /// <param name="languageProvider"></param>
        /// <param name="requiresArguments"></param>
        /// <param name="providesArguments"></param>
        protected override void GeneratePostInitializationCode(string directiveName, StringBuilder codeBuffer, CodeDomProvider languageProvider, IDictionary<string, string> requiresArguments, IDictionary<string, string> providesArguments)
        {
            if ((StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0))
            {
                IList<string> fileNames = new System.Collections.ObjectModel.Collection<string>();
                LoadInitializationModelFileNames(fileNames);


                if (requiresArguments["folderName"] != null)
                {
                    IList<string> fileExtentions = new string[] { ".sd", ".ad", ".sdm", ".ldd" };
                    foreach (String fName in System.IO.Directory.GetFiles(requiresArguments["folderName"]))
                    {
                        if (fileExtentions.Contains(System.IO.Path.GetExtension(fName)))
                            fileNames.Add(fName);
                    }
                }
                else
                {
                    if (requiresArguments["fileName"] == null || requiresArguments["filename"] == "")
                        throw new ArgumentException("File name is empty");

                    fileNames.Add(requiresArguments["fileName"]);
                }


                //CodeMethodInvokeExpression beginTran = new CodeMethodInvokeExpression(
                //    new CodePropertyReferenceExpression(
                //        new CodePropertyReferenceExpression(new CodeThisReferenceExpression(), "Store"), "TransactionManager"),
                //    "BeginTransaction",
                //    new CodeExpression[] { new CodePrimitiveExpression("ProcessorLoad"), new CodePrimitiveExpression(true) });
                    
                //CodeTryCatchFinallyStatement tryCatch = new CodeTryCatchFinallyStatement(
                //    new CodeStatement[]{beginTran

                foreach (String filename in fileNames)
                {
                    CodeMethodInvokeExpression invokeExpression = new CodeMethodInvokeExpression(
                        new CodeMethodReferenceExpression(new CodeTypeReferenceExpression(typeof(ApplicationDesignerFileLoader)), "Load"), new CodePropertyReferenceExpression(new CodeThisReferenceExpression(), "Errors"), new CodePropertyReferenceExpression(new CodeThisReferenceExpression(), "Store"), new CodePrimitiveExpression(filename));
                    CodeAssignStatement assign = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), providesArguments["SolutionModelRoot"].ToLower(CultureInfo.InvariantCulture) + "Value"), invokeExpression);

                    CodeGeneratorOptions options = new CodeGeneratorOptions();
                    options.BlankLinesBetweenMembers = true;
                    options.IndentString = "    ";
                    options.VerbatimOrder = true;
                    options.BracingStyle = "C";
                    using (StringWriter writer = new StringWriter(codeBuffer, CultureInfo.InvariantCulture))
                    {
                        languageProvider.GenerateCodeFromStatement(assign, writer, options);
                    }
                }
            }
        }

        protected override void PostProcessArguments(string directiveName, IDictionary<string, string> requiresArguments, IDictionary<string, string> providesArguments)
        {
            if ((StringComparer.InvariantCultureIgnoreCase.Compare(directiveName, DIRECTIVE_NAME) == 0))
            {
                if (requiresArguments["fileName"] != null && requiresArguments["fileName"] != "")
                {
                    // Give the host a chance to resolve the fileName
                    requiresArguments["fileName"] = this.Host.ResolvePath(requiresArguments["fileName"]);
                }
            }
        }

        public override string[] GetImportsForProcessingRun()
        {
            List<string> imports = new List<string>(base.GetImportsForProcessingRun());

            imports.Add("Microsoft.VisualStudio.Modeling");
            imports.Add("System.CodeDom.Compiler");
            imports.Add("Microsoft.VisualStudio.EnterpriseTools.Application.Modeling");
            imports.Add("Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling");
            
            return imports.ToArray();
        }

        public override string[] GetReferencesForProcessingRun()
        {
            List<string> references = new List<string>(base.GetReferencesForProcessingRun());

            // returning assembly strong names causes paths to these assemblies to be retrieved from the GAC.
            references.Add("Microsoft.VisualStudio.Modeling, Version=8.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a");
            references.Add("Microsoft.VisualStudio.Modeling.Diagrams, Version=8.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a");
            references.Add(this.GetType().Assembly.Location);
            references.Add("Microsoft.VisualStudio.EnterpriseTools.SdmDesigners, Version=8.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a");
            references.Add("Microsoft.VisualStudio.EnterpriseTools.TypeSystem, Version=8.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a");
            references.Add(typeof(ApplicationDesignerFileLoader).Assembly.Location);
            references.Add(typeof(Microsoft.VisualStudio.TextTemplating.DirectiveProcessor).Assembly.Location);
            references.Add(typeof(Eastbanc.VSIP.TextTemplating.VSHost.ModelingTextTransformation).Assembly.Location);
            
            
            return references.ToArray();
        }
    }
}
