using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Modeling.Diagnostics;
using System.CodeDom.Compiler;
using Microsoft.VisualStudio.Modeling;
using System.IO;
using System.Collections;
using System.Xml;
using Microsoft.VisualStudio.EnterpriseTools.Sdm;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Serialization;
using Microsoft.VisualStudio.EnterpriseTools.Application.Modeling;
using Microsoft.VisualStudio.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.LogicalInfrastructureDesign.Modeling;
using Microsoft.VisualStudio.EnterpriseTools.LogicalInfrastructureDesign.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling.Diagrams;
using Microsoft.VisualStudio.EnterpriseTools.ArtifactModel.Project;
using Microsoft.VisualStudio.EnterpriseTools.ArtifactModel.Asmx;
using Microsoft.VisualStudio.EnterpriseTools.ArtifactModel.Clr;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Serialization.Mapping;
using Microsoft.SystemDefinitionModel.Serialize;
using Microsoft.VisualStudio.EnterpriseTools.Sdm.Serialization.Resolution;

namespace Eastbanc.VSIP.Sdm
{
    /// <summary>
    /// Runtime helper class for the Directive processor
    /// </summary>
    public static class ApplicationDesignerFileLoader
    {
        public static MemoryStream DumpXmlContents(string xmlContents)
        {
            MemoryStream stream1 = new MemoryStream();
            XmlTextWriter writer1 = null;
            try
            {
                writer1 = new XmlTextWriter(stream1, new UTF8Encoding(true));
                writer1.Formatting = Formatting.Indented;
                writer1.WriteStartDocument();
                writer1.WriteRaw(xmlContents);
            }
            finally
            {
                if (writer1 != null)
                {
                    writer1.Flush();
                }
                if (stream1 != null)
                {
                    stream1.Seek((long)0, SeekOrigin.Begin);
                }
            }
            return stream1;
        }


        private static void SdmElementDeserializationComplete(object sender, SdmElementDeserializationCompleteEventArgs e)
        {
            SoftwareComponentDocument document1 = e.RootElement as SoftwareComponentDocument;
            if (document1 != null)
            {
                //SoftwareComponentType type1 = Helpers.GetSoftwareComponentType(document1);
                //if ((type1 != null) && ((document1.VSProject == null) || !ArtifactMappingEngine.get_ModelMapper().IsModelElementMappedToArtifact((ModelElement)document1.VSProject)))
                //{
                //    foreach (Port port1 in ((IEnumerable)type1.Ports))
                //    {
                //        WebServiceProviderPort port2 = port1 as WebServiceProviderPort;
                //        if (port2 != null)
                //        {
                //            ClrClass class1 = port2.ClrClass;
                //            if (class1 != null)
                //            {
                //                class1.ReconcileClrTypeRefs();
                //            }
                //        }
                //    }
                //}
            }
        }

        
        public static void LoadExistingDocument(string fileName, Stream fileStream, Store store)
        {
            if (fileStream != null)
            {
                SdmSerializer serializer1 = new SdmSerializer();

                //MappingNode mapping1 = new TestMappingResolver();
                //SdmMapping.Register(mapping1, typeof(Component));
                                
                serializer1.SdmElementDeserializationComplete += new SdmElementDeserializationCompleteEventHandler(SdmElementDeserializationComplete);
                serializer1.MaxErrors = 10;
                XmlTextReader reader1 = new XmlTextReader(fileStream);
                MemoryStream stream1 = null;
                SolutionModelRoot solutionRoot;

                try
                {
                    if (reader1.IsStartElement("Whitehorse"))
                    {
                        reader1.ReadStartElement("Whitehorse");
                        System.Text.Encoding encoding1 = reader1.Encoding;
                        reader1.ReadStartElement("Solution");
                        try
                        {
                            stream1 = DumpXmlContents(reader1.ReadOuterXml());
                            LoadSdmFile(fileName, stream1, store, serializer1, true);

                            solutionRoot = SolutionModelRoot.GetSolutionModelRoot(store);
                        }
                        finally
                        {
                            if (stream1 != null)
                            {
                                stream1.Close();
                            }
                        }

                        reader1.ReadEndElement();

                        while (!reader1.EOF)
                        {
                            try
                            {
                                reader1.ReadStartElement("SdmDocument");
                            }
                            catch (XmlException)
                            {
                                break;
                            }
                            try
                            {
                                stream1 = DumpXmlContents(reader1.ReadOuterXml());
                                LoadSdmFile(fileName, stream1, store, serializer1, false);
                            }
                            finally
                            {
                                if (stream1 != null)
                                {
                                    stream1.Close();
                                }
                            }
                            reader1.ReadEndElement();
                        }
                    }
                    else
                    {
                        try
                        {
                            stream1 = DumpXmlContents(reader1.ReadOuterXml());
                            LoadSdmFile(fileName, stream1, store, serializer1, false);
                        }
                        finally
                        {
                            if (stream1 != null)
                            {
                                stream1.Close();
                            }
                        }
                    }
                }
                finally
                {
                    if (stream1 != null)
                    {
                        stream1.Close();
                    }
                }
            }
        }


        public static bool LoadSdmFile(string fileName, Stream stream, Store store, SdmSerializer sdmSerializer, bool setRootElement)
        {
            if ((stream == null) || (sdmSerializer == null))
            {
                return false;
            }
            bool flag1 = true;
            
            bool flag2 = sdmSerializer.Load(stream, store);
                       
            if (!flag2)
            {
                SdmSerializerMessage[] msg = sdmSerializer.GetMessages();
                string text1 = sdmSerializer.GetFormattedMessages();
                
                object[] objArray1 = new object[] { fileName, text1 };

                System.Diagnostics.Debug.WriteLine(text1);
                //throw new Exception(text1, msg[0].Exception);
            }

            if (store.RuleManager.IsRuleSuspended || store.TransactionManager.CurrentTransaction.ForceAllRulesToCommitTime)
            {
                store.ResolveReferences();
            }
            
            return flag2;
        }


        public static SolutionModelRoot GetSolutionModelRoot(Store store)
        {
            //if (((SolutionModelRoot.storeCacheReference != null) && SolutionModelRoot.storeCacheReference.IsAlive) && ((SolutionModelRoot.storeCacheReference.Target == store) && SolutionModelRoot.modelRootCacheReference.IsAlive))
            //{
            //    return (SolutionModelRoot)SolutionModelRoot.modelRootCacheReference.Target;
            //}
            //if (store == null)
            //{
            //    return null;
            //}
            //IList list1 = store.ElementDirectory.GetElements(SolutionModelRoot.MetaClassGuid);
            //if (list1.Count == 0)
            //{
            //    SolutionModelRoot.modelRootCacheReference = null;
            //    SolutionModelRoot.storeCacheReference = null;
            //    return null;
            //}
            //SolutionModelRoot root1 = (SolutionModelRoot)list1[0];
            //SolutionModelRoot.modelRootCacheReference = new WeakReference(root1);
            //SolutionModelRoot.storeCacheReference = new WeakReference(store);
            return SolutionModelRoot.GetSolutionModelRoot(store);
        }
        
        /// <summary>
        /// Load a SchemaDefenitionLanguage model
        /// </summary>
        /// <param name="store"></param>
        /// <param name="inputFile"></param>
        /// <returns></returns>
        public static SolutionModelRoot Load(CompilerErrorCollection errors, Store store, string inputFile)
        {
            if (errors == null) throw new ArgumentNullException("errors");
            if (store == null) throw new ArgumentNullException("store");
            if (inputFile == null) throw new ArgumentNullException("inputFile");

            TransactionLog trLog = new TransactionLog();

            if (System.IO.File.Exists(inputFile))
            {
                try
                {
                    using (System.IO.FileStream fileStream = File.OpenRead(inputFile))
                    {
                        using (Transaction t = store.TransactionManager.BeginTransaction("ProcessorLoad", true))
                        {
                            store.RegisterTransactionLog(trLog);
                            store.RuleManager.SuspendRuleNotification();
                            SolutionModelRoot.CreateSolutionModelRoot(store);

                            LoadExistingDocument(inputFile, fileStream, store);

                            store.ResolveReferences();

                            t.Commit();
                        }

                        store.ResolveReferences();

                        store.RuleManager.ResumeRuleNotification();

                        UpdateDocumentReferences(SolutionModelRoot.GetSolutionModelRoot(store));
                        
                    }
                }
                catch (System.IO.FileLoadException ex)
                {
                    throw ex;
                }
                catch (System.IO.FileNotFoundException ex)
                {
                    throw ex;
                }
            }
            
            //SolutionModelRoot _SolutionModelRoot = SolutionModelRoot.GetSolutionModelRoot(store);
            //Store _store = _SolutionModelRoot.Store;
            
            //IList list = store.ElementDirectory.GetElements(Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.PlugIns.WindowsComponent.MetaClassGuid);
            //list = store.ElementDirectory.GetElements(Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.PlugIns.AspComponent.MetaClassGuid);
            //list = store.ElementDirectory.GetElements(Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationComponent.MetaClassGuid);

            //list = store.ElementDirectory.GetElements(Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationComponent.MetaClassGuid);

            //System.Collections.Generic.List<string> systemList = new System.Collections.Generic.List<string>();

            //EventHandler ee = delegate(object o, System.EventArgs e1)
            //{
            //    object[] oo = o as object[];
            //    Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationCompoundComponentType component = oo[0] as Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationCompoundComponentType;

            //    //WriteLine(String.Format("<wix:Feature Id='{0}' Level='1' Title='{1}' Description='{2}'>", 1, component.Name, component.FullyQualifiedName));
            //    if (component != null)
            //    {
            //        foreach (Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling.Component c in component.Components)
            //        {
            //            if (c.MetaClass.Name.Equals("ApplicationCompoundComponent") && !systemList.Contains(c.Name))
            //            {
            //                systemList.Add(c.Name);

            //                object[] cc1 = new object[] { c.ComponentType as ApplicationCompoundComponentType, oo[1] };
            //                ((EventHandler)oo[1])(cc1, e1);

                            
            //            }
            //        }
            //    }

            //    WriteLine("</wix:Feature>");
            //};


            foreach (Component app in store.ElementDirectory.GetElements(Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.PlugIns.AspComponent.MetaClassGuid))
            {
                string mname = app.Name;
            }

            
            return SolutionModelRoot.GetSolutionModelRoot(store);
        }

        private static void WriteLine(string p)
        {
            //EventHandler e = delegate(object o, System.EventArgs e1)
            //{
            //    object[] oo = o as object[];
            //    Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationCompoundComponentType component = oo[0] as Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationCompoundComponentType;

            //    foreach (Microsoft.VisualStudio.EnterpriseTools.Sdm.Modeling.Component c in component.Components)
            //    {
            //        if (c.MetaClass.Name.Equals("ApplicationCompoundComponent"))
            //        {
            //            Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationCompoundComponentType cc = SolutionModelRoot.GetSolutionModelRoot(c.Store).FindSdmElement(c.FullyQualifiedName) as Microsoft.VisualStudio.EnterpriseTools.Application.Modeling.ApplicationCompoundComponentType;
            //            WriteLine(c.Name);

            //            object[] cc1 = new object[] { cc, oo[1] };
            //            ((EventHandler)oo[1])(cc1, e1);
            //        }
            //    }
            //};

            //object[] cc2 = new object[] { cc, e };

            //e(cc2, null);

            //throw new Exception("The method or operation is not implemented.");
            System.Diagnostics.Debug.WriteLine(p);
        }

        private static void WriteLine(string p, string p_2)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public static void UpdateDocumentReferences(SolutionModelRoot solutionModelRoot)
        {
            if (solutionModelRoot != null)
            {
                foreach (SdmDocumentReference reference2 in ((IEnumerable)solutionModelRoot.SdmDocumentReferences))
                {
                    SdmDocumentReference.SdmDocumentReferenceReferencedSdmDocumentIdFieldHandler.Instance.Resolve((ModelElement)reference2);
                }
            }
        }
    }

        public class TestMappingResolver: MappingResolver
        {
   
             // Methods
            public TestMappingResolver() 
            {
            }

            public override Type DetermineType(object sdmXmlElement, Type designDataType, SdmSerializationContext context)
            {
                SystemDefinition definition1 = sdmXmlElement as SystemDefinition;
                if (definition1 == null)
                {
                    return null;
                }
                if (definition1.AbstractSpecified && definition1.Abstract)
                {
                    return typeof(AbstractComponentType);
                }
                return typeof(ComponentType);
            }
        }



    

}
