namespace Eastbanc.VSIP.TextTemplating.VSHost
{
    using System;
    using System.Diagnostics;
    using System.Runtime.InteropServices;

    public abstract class BaseCodeGenerator : IVsSingleFileGenerator, IDisposable
    {
        protected BaseCodeGenerator()
        {
            this.codeFileNameSpace = string.Empty;
            this.codeFilePath = string.Empty;
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            this.codeGeneratorProgress = null;
        }

        ~BaseCodeGenerator()
        {
            this.Dispose(false);
        }

        public void Generate(string wszInputFilePath, string bstrInputFileContents, string wszDefaultNamespace, out IntPtr rgbOutputFileContents, out int pcbOutput, IVsGeneratorProgress pGenerateProgress)
        {
            if (bstrInputFileContents == null)
            {
                throw new ArgumentNullException(bstrInputFileContents);
            }
            this.codeFilePath = wszInputFilePath;
            this.codeFileNameSpace = wszDefaultNamespace;
            this.codeGeneratorProgress = pGenerateProgress;
            byte[] buffer1 = this.GenerateCode(wszInputFilePath, bstrInputFileContents);
            if (buffer1 == null)
            {
                rgbOutputFileContents = IntPtr.Zero;
                pcbOutput = 0;
            }
            else
            {
                pcbOutput = buffer1.Length;
                rgbOutputFileContents = Marshal.AllocCoTaskMem(pcbOutput);
                Marshal.Copy(buffer1, 0, rgbOutputFileContents, pcbOutput);
            }
        }

        protected abstract byte[] GenerateCode(string inputFileName, string inputFileContent);
        protected virtual void GeneratorErrorCallback(bool warning, int level, string message, int line, int column)
        {
            IVsGeneratorProgress progress1 = this.CodeGeneratorProgress;
            if (progress1 != null)
            {
                if (line > 0)
                {
                    line--;
                }
                progress1.GeneratorError(warning, level, message, line, column);
            }
        }

        public abstract string GetDefaultExtension();

        internal IVsGeneratorProgress CodeGeneratorProgress
        {
            [DebuggerStepThrough]
            get
            {
                return this.codeGeneratorProgress;
            }
        }

        protected string FileNameSpace
        {
            [DebuggerStepThrough]
            get
            {
                return this.codeFileNameSpace;
            }
        }

        protected string InputFilePath
        {
            [DebuggerStepThrough]
            get
            {
                return this.codeFilePath;
            }
        }


        private string codeFileNameSpace;
        private string codeFilePath;
        private IVsGeneratorProgress codeGeneratorProgress;
    }
}

