namespace Eastbanc.VSIP.TextTemplating.VSHost
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Runtime.InteropServices;
    using System.Text;

    public abstract class BaseCodeGeneratorWithSite : BaseCodeGenerator, IObjectWithSite
    {
        protected BaseCodeGeneratorWithSite()
        {
        }

        protected virtual string CreateExceptionMessage(Exception exception)
        {
            StringBuilder builder1 = new StringBuilder((exception.Message != null) ? exception.Message : string.Empty);
            for (Exception exception1 = exception.InnerException; exception1 != null; exception1 = exception1.InnerException)
            {
                string text1 = exception1.Message;
                if ((text1 != null) && (text1.Length > 0))
                {
                    builder1.AppendLine(" " + text1);
                }
            }
            return builder1.ToString();
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                if (this.serviceProvider != null)
                {
                    this.serviceProvider.Dispose();
                    this.serviceProvider = null;
                }
            }
            finally
            {
                base.Dispose(disposing);
            }
        }

        protected object GetService(Guid serviceGuid)
        {
            return this.SiteServiceProvider.GetService(serviceGuid);
        }

        protected object GetService(Type serviceType)
        {
            return this.SiteServiceProvider.GetService(serviceType);
        }

        public virtual void GetSite(ref Guid riid, out IntPtr ppvSite)
        {
            if (this.site == null)
            {
                throw new COMException(Resources.ObjectNotSited, -2147467259);
            }
            IntPtr ptr1 = Marshal.GetIUnknownForObject(this.site);
            IntPtr ptr2 = IntPtr.Zero;
            Marshal.QueryInterface(ptr1, ref riid, out ptr2);
            if (ptr2 == IntPtr.Zero)
            {
                throw new COMException(Resources.SiteInterfaceNotSupported, -2147467262);
            }
            ppvSite = ptr2;
        }

        public virtual void SetSite(object pUnkSite)
        {
            this.site = pUnkSite;
            this.serviceProvider = null;
        }

        protected void SetWaitCursor()
        {
            if (this.site != null)
            {
                IVsHierarchy hierarchy1 = this.GetService(typeof(IVsHierarchy)) as IVsHierarchy;
                if (hierarchy1 != null)
                {
                    Microsoft.VisualStudio.OLE.Interop.IServiceProvider provider1 = null;
                    if (hierarchy1.GetSite(out provider1) == 0)
                    {
                        IntPtr ptr1;
                        Guid guid1 = typeof(SVsUIShell).GUID;
                        Guid guid2 = typeof(IVsUIShell).GUID;
                        if (provider1.QueryService(ref guid1, ref guid2, out ptr1) == 0)
                        {
                            IVsUIShell shell1 = Marshal.GetObjectForIUnknown(ptr1) as IVsUIShell;
                            if (shell1 != null)
                            {
                                shell1.SetWaitCursor();
                            }
                        }
                    }
                }
            }
        }


        [CLSCompliant(false)]
        protected DTE Dte
        {
            get
            {
                DTE dte1 = null;
                IVsHierarchy hierarchy1 = this.GetService(typeof(IVsHierarchy)) as IVsHierarchy;
                if (hierarchy1 != null)
                {
                    Microsoft.VisualStudio.OLE.Interop.IServiceProvider provider1 = null;
                    hierarchy1.GetSite(out provider1);
                    if (provider1 != null)
                    {
                        Guid guid1 = typeof(DTE).GUID;
                        IntPtr ptr1 = IntPtr.Zero;
                        ErrorHandler.ThrowOnFailure(provider1.QueryService(ref guid1, ref guid1, out ptr1));
                        if (ptr1 != IntPtr.Zero)
                        {
                            dte1 = Marshal.GetObjectForIUnknown(ptr1) as DTE;
                        }
                    }
                }
                return dte1;
            }
        }

        [CLSCompliant(false)]
        protected IVsErrorList ErrorList
        {
            get
            {
                IVsErrorList list1 = null;
                IVsHierarchy hierarchy1 = this.GetService(typeof(IVsHierarchy)) as IVsHierarchy;
                if (hierarchy1 != null)
                {
                    Microsoft.VisualStudio.OLE.Interop.IServiceProvider provider1 = null;
                    hierarchy1.GetSite(out provider1);
                    if (provider1 != null)
                    {
                        Guid guid1 = typeof(SVsErrorList).GUID;
                        Guid guid2 = typeof(IVsErrorList).GUID;
                        IntPtr ptr1 = IntPtr.Zero;
                        ErrorHandler.ThrowOnFailure(provider1.QueryService(ref guid1, ref guid2, out ptr1));
                        if (ptr1 != IntPtr.Zero)
                        {
                            list1 = Marshal.GetObjectForIUnknown(ptr1) as IVsErrorList;
                        }
                    }
                }
                return list1;
            }
        }

        private ServiceProvider SiteServiceProvider
        {
            get
            {
                if (this.serviceProvider == null)
                {
                    Microsoft.VisualStudio.OLE.Interop.IServiceProvider provider1 = this.site as Microsoft.VisualStudio.OLE.Interop.IServiceProvider;
                    this.serviceProvider = new ServiceProvider(provider1);
                }
                return this.serviceProvider;
            }
        }


        private ServiceProvider serviceProvider;
        private object site;

        #region IObjectWithSite Members

        void IObjectWithSite.GetSite(ref Guid riid, out IntPtr ppvSite)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        void IObjectWithSite.SetSite(object pUnkSite)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}

