namespace Eastbanc.VSIP.TextTemplating.VSHost
{
    using Microsoft.VisualStudio.Shell;
    using System;
    using System.Diagnostics;
    using System.Globalization;
    using System.IO;

    [CLSCompliant(false), AttributeUsage(AttributeTargets.Class, AllowMultiple=true)]
    public sealed class ProvideIncludeFolderAttribute : RegistrationAttribute
    {
        public ProvideIncludeFolderAttribute(string extension, int index, string folder)
        {
            if (string.IsNullOrEmpty(extension))
            {
                throw new ArgumentNullException("extension");
            }
            if (!extension.StartsWith(".", StringComparison.CurrentCulture))
            {
                throw new ArgumentOutOfRangeException("extension", Resources.ExtensionMalformed);
            }
            if (index < 0)
            {
                throw new ArgumentOutOfRangeException("index", Resources.IndexOutOfRange);
            }
            this.extension = extension;
            this.index = index;
            this.folder = folder;
        }

        public override void Register(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            string text1 = @"TextTemplating\IncludeFolders\" + this.Extension;
            using (RegistrationAttribute.Key key1 = context.CreateKey(text1))
            {
                if (key1 != null)
                {
                    string text2 = context.EscapePath(Path.Combine(context.ComponentPath, this.Folder));
                    key1.SetValue(string.Format(CultureInfo.InvariantCulture, "Include{0}", new object[] { this.Index }), text2);
                    context.Log.WriteLine(string.Format(CultureInfo.CurrentCulture, Resources.ProvideIncludeFolderLogRegistered, new object[] { this.Extension, this.Index, text2 }));
                }
            }
        }

        public override void Unregister(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            context.RemoveValue(@"TextTemplating\IncludeFolders\" + this.Extension, string.Format(CultureInfo.InvariantCulture, "Include{0}", new object[] { this.Index }));
            context.Log.WriteLine(string.Format(CultureInfo.CurrentCulture, Resources.ProvideIncludeFolderLogUnregistered, new object[] { this.Extension, this.Index, this.Folder }));
        }


        public string Extension
        {
            [DebuggerStepThrough]
            get
            {
                return this.extension;
            }
        }

        public string Folder
        {
            [DebuggerStepThrough]
            get
            {
                return this.folder;
            }
        }

        public int Index
        {
            [DebuggerStepThrough]
            get
            {
                return this.index;
            }
        }


        private string extension;
        private string folder;
        public const string IncludeFoldersKeyName = "IncludeFolders";
        private int index;
        public const string TextTemplatingKeyName = "TextTemplating";
    }
}

