namespace Eastbanc.VSIP.TextTemplating.VSHost
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Text;

    [Guid("B9C665D2-0451-4ae2-A38C-443B85D634F2")]
    public class TemplatedCodeGenerator : BaseCodeGeneratorWithSite, ITextTemplatingCallback
    {
        public TemplatedCodeGenerator()
        {
            this.extension = ".cs";
        }

        public void ErrorCallback(bool warning, string message, int line, int column)
        {
            if (line == -1)
            {
                column = -1;
            }
            this.GeneratorErrorCallback(warning, 1, message, line, column);
            this.errors = true;
        }

        protected override byte[] GenerateCode(string inputFileName, string inputFileContent)
        {
            base.SetWaitCursor();
            this.errors = false;
            ITextTemplating templating1 = this.TextTemplating;
            string text1 = "";
            if (templating1 != null)
            {
                text1 = templating1.ProcessTemplate(inputFileName, inputFileContent, this);
            }
            else
            {
                this.GeneratorErrorCallback(false, 100, Resources.TextTemplatingUnavailable, -1, -1);
                text1 = Resources.FallbackCodeGenOutput;
                this.errors = true;
            }
            if (this.errors)
            {
                IVsErrorList list1 = base.ErrorList;
                if (list1 != null)
                {
                    try
                    {
                        list1.BringToFront();
                        list1.ForceShowErrors();
                    }
                    catch
                    {
                    }
                }
            }
            Encoding encoding1 = Encoding.Default;
            try
            {
                using (FileStream stream1 = new FileStream(inputFileName, FileMode.Open, FileAccess.Read))
                {
                    if (stream1.Length > 0)
                    {
                        using (StreamReader reader1 = new StreamReader(stream1, true))
                        {
                            char[] chArray1 = new char[1];
                            reader1.Read(chArray1, 0, 1);
                            encoding1 = reader1.CurrentEncoding;
                            reader1.BaseStream.Position = 0;
                            if (encoding1 == Encoding.UTF8)
                            {
                                byte[] buffer1 = encoding1.GetPreamble();
                                if (stream1.Length >= buffer1.Length)
                                {
                                    byte[] buffer2 = new byte[buffer1.Length];
                                    stream1.Read(buffer2, 0, buffer2.Length);
                                    for (int num1 = 0; num1 < buffer2.Length; num1++)
                                    {
                                        if (buffer2[num1] != buffer1[num1])
                                        {
                                            encoding1 = Encoding.Default;
                                            break;
                                        }
                                    }
                                }
                                else
                                {
                                    encoding1 = Encoding.Default;
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception)
            {
            }
            if (encoding1 == null)
            {
                encoding1 = Encoding.UTF8;
            }
            byte[] buffer3 = encoding1.GetBytes(text1);
            byte[] buffer4 = encoding1.GetPreamble();
            if ((buffer4 != null) && (buffer4.Length > 0))
            {
                bool flag1 = false;
                if (buffer3.Length >= buffer4.Length)
                {
                    flag1 = true;
                    for (int num2 = 0; num2 < buffer4.Length; num2++)
                    {
                        if (buffer4[num2] != buffer3[num2])
                        {
                            flag1 = false;
                            break;
                        }
                    }
                }
                if (!flag1)
                {
                    byte[] buffer5 = new byte[buffer4.Length + buffer3.Length];
                    buffer4.CopyTo(buffer5, 0);
                    buffer3.CopyTo(buffer5, buffer4.Length);
                    buffer3 = buffer5;
                }
            }
            return buffer3;
        }

        public override string GetDefaultExtension()
        {
            return this.extension;
        }

        public void SetFileExtension(string extension)
        {
            if (string.IsNullOrEmpty(extension))
            {
                throw new ArgumentNullException("extension");
            }
            this.extension = extension;
        }


        protected ITextTemplating TextTemplating
        {
            get
            {
                ITextTemplating templating1 = null;
                IVsHierarchy hierarchy1 = base.GetService(typeof(IVsHierarchy)) as IVsHierarchy;
                if (hierarchy1 != null)
                {
                    Microsoft.VisualStudio.OLE.Interop.IServiceProvider provider1 = null;
                    hierarchy1.GetSite(out provider1);
                    if (provider1 != null)
                    {
                        Guid guid1 = typeof(STextTemplating).GUID;
                        IntPtr ptr1 = IntPtr.Zero;
                        ErrorHandler.ThrowOnFailure(provider1.QueryService(ref guid1, ref guid1, out ptr1));
                        if (ptr1 != IntPtr.Zero)
                        {
                            templating1 = Marshal.GetObjectForIUnknown(ptr1) as ITextTemplating;
                        }
                    }
                }
                return templating1;
            }
        }


        private bool errors;
        private string extension;
    }
}

