/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using Microsoft.VisualStudio.TextManager.Interop;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Designer.Interfaces;
using Microsoft.VisualStudio.Shell;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Diagnostics;
using Microsoft.Win32;
using System.Globalization;
using System.IO;
using System.Collections;
using System.Xml;
using System.Text;
using System.Net;
using IOleServiceProvider = Microsoft.VisualStudio.OLE.Interop.IServiceProvider;
using IServiceProvider = System.IServiceProvider;
using ShellConstants = Microsoft.VisualStudio.Shell.Interop.Constants;
using OleConstants = Microsoft.VisualStudio.OLE.Interop.Constants;

namespace Microsoft.VisualStudio.Package
{
	//The tracker class will need to implement helpers to CALL IVsTrackProjectDocuments2 for notifications
	internal class TrackDocumentsHelper
	{
		private ProjectNode theProject;


		public TrackDocumentsHelper(ProjectNode project)
		{
			this.theProject = project;
		}

		private IVsTrackProjectDocuments2 GetIVsTrackProjectDocuments2()
		{
			Debug.Assert(this.theProject != null && !this.theProject.IsClosed && this.theProject.Site != null);
			
			IVsTrackProjectDocuments2 documentTracker = this.theProject.Site.GetService(typeof(SVsTrackProjectDocuments)) as IVsTrackProjectDocuments2;
			if (documentTracker == null)
			{
				throw new InvalidOperationException();
			}
			
			return  documentTracker;
		}


		public bool CanAddItems(string[] files, VSQUERYADDFILEFLAGS[] flags)
		{
			if (files == null || files.Length == 0)
			{
				return false;
			}

			int len = files.Length;
			VSQUERYADDFILERESULTS[] summary = new VSQUERYADDFILERESULTS[1];
			ErrorHandler.ThrowOnFailure(this.GetIVsTrackProjectDocuments2().OnQueryAddFiles(this.theProject, len, files, flags, summary, null));
			if (summary[0] == VSQUERYADDFILERESULTS.VSQUERYADDFILERESULTS_AddNotOK)
			{
				return false;
			}
			return true;
		}


		/// <summary>
		/// Gets called when an item is added.
		/// </summary>
		public void OnItemAdded(string file,  VSADDFILEFLAGS flag)
		{
			ErrorHandler.ThrowOnFailure(this.GetIVsTrackProjectDocuments2().OnAfterAddFilesEx(this.theProject, 1, new string[1] { file }, new VSADDFILEFLAGS[1] { flag }));
		}

		/// <summary>
		///  Get's called to tell the env that a file can be deleted.
		/// </summary>
		public bool CanRemoveItems(string[] files, VSQUERYREMOVEFILEFLAGS[] flags)
		{
			if (files == null || files.Length == 0)
			{
				return false;
			}
			int length = files.Length;

			VSQUERYREMOVEFILERESULTS[] summary = new VSQUERYREMOVEFILERESULTS[1];

			ErrorHandler.ThrowOnFailure(this.GetIVsTrackProjectDocuments2().OnQueryRemoveFiles(this.theProject, length, files, flags, summary, null));
			if (summary[0] == VSQUERYREMOVEFILERESULTS.VSQUERYREMOVEFILERESULTS_RemoveNotOK)
			{
				return false;
			}

			return true;
		}


		/// <summary>
		/// Gets called when an item is removed.
		/// </summary>
		public void OnItemRemoved(string file, VSREMOVEFILEFLAGS flag)
		{
			ErrorHandler.ThrowOnFailure(this.GetIVsTrackProjectDocuments2().OnAfterRemoveFiles(this.theProject, 1, new string[1] { file }, new VSREMOVEFILEFLAGS[1] { flag }));
		}
		
		/// <summary>
		/// Get's called to ask the environent if a file is allowed to be renamed
		/// </summary>
		/// returns FALSE if the doc can not be renamed
		public bool CanRenameItem(string strOldName, string strNewName, VSRENAMEFILEFLAGS flag)
		{
			int iCanContinue = 0;
			ErrorHandler.ThrowOnFailure(this.GetIVsTrackProjectDocuments2().OnQueryRenameFile(this.theProject, strOldName, strNewName, flag, out iCanContinue));
			return (iCanContinue != 0);
		}

		/// <summary>
		/// Get's called to tell the env that a file was renamed
		/// </summary>
		/// 
		public void OnItemRenamed(string strOldName, string strNewName, VSRENAMEFILEFLAGS flag)
		{
			ErrorHandler.ThrowOnFailure(this.GetIVsTrackProjectDocuments2().OnAfterRenameFile(this.theProject, strOldName, strNewName, flag));
		}		
	}
}

