using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Package;
using System.Diagnostics;
using Microsoft.VisualStudio;
using System.Runtime.InteropServices;

namespace Eastbanc.VSIP.Sdm.VSProject
{
    /// <include file='doc\PropertyPages.uex' path='docs/doc[@for="PrototypePropertyPage"]/*' />
    [ComVisible(true), Guid("2A5A18ED-DD4F-4211-AF3B-C7DC9B7C00E1")]
    public class PrototypePropertyPage : SettingsPage
    {
        private String prototypeTypeName;
        private PrototypeType prototypeType;
        private string prototypeOutput;
        private string prototypeName;
        private PrototypeLayer prototypeLayer;
        private string prototypeDocument;
        private String prototypeCaption;
        private String prototypeBitmap;

        public PrototypePropertyPage()
        {
            this.Name = Resources.PrototypeCaption;
        }

        /// <include file='doc\PropertyPages.uex' path='docs/doc[@for="GeneralPropertyPage.GetClassName"]/*' />
        public override string GetClassName()
        {
            return this.GetType().FullName;
        }

        /// <include file="doc\PropertyPages.uex" path="docs/doc[@for=&quot;GeneralPropertyPage.PrototypeType&quot;]/*"/>
        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeTypeName")]
        [SRDescriptionAttribute("Required. The name of the SDM type")]
        public String PrototypeTypeName
        {
            get { return prototypeTypeName; }
            set { prototypeTypeName = value; this.IsDirty = true; }
        }

        /// <include file="doc\PropertyPages.uex" path="docs/doc[@for=&quot;GeneralPropertyPage.PrototypeType&quot;]/*"/>
        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeType")]
        [SRDescriptionAttribute("PrototypeType")]
        public PrototypeType PrototypeType
        {
            get { return prototypeType; }
            set { prototypeType = value; this.IsDirty = true; }
        }

        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeOutput")]
        [SRDescriptionAttribute("")]
        public string PrototypeOutput
        {
            get { return prototypeOutput; }
        }

        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeName")]
        [SRDescriptionAttribute("Optional. The name of the prototype. The name is used by Visual Studio as the F1 Help keyword for the Toolbox item. This value identifies the prototype in the Toolbox and must be unique from any other prototype name in the Toolbox. This value defaults to the <sdmTypeName> value specified. ")]
        public string PrototypeName
        {
            get { return prototypeName; }
            set { prototypeName = value; this.IsDirty = true; }
        }

        /// <include file="doc\PropertyPages.uex" path="docs/doc[@for=&quot;GeneralPropertyPage.PrototypeType&quot;]/*"/>
        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeLayer")]
        [SRDescriptionAttribute(@"If the /Type switch is used: Required. The layer for which the prototype is being generated. Specify a value of Application for an Application Designer prototype or a Host for a Logical Datacenter Designer prototype.
This value defaults to the layer that is specified in <typeName>. If no layer is specified by <sdmType> or by the /Layer option, an error is returned.
If the /File switch is used: Optional. The default layer for SDM types (endpoints or systems) whose layer is not explicitly specified in the .sdmdocument file. If this switch is not used, the prototypes are not generated for types that are missing the Layer attribute. Valid values: Application or Host. ")]
        public PrototypeLayer PrototypeLayer
        {
            get { return prototypeLayer; }
            set { prototypeLayer = value; this.IsDirty = true; }
        }
        
        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeDocument")]
        [SRDescriptionAttribute("Required. The identity of the .sdm file that contains the SDM type.")]
        public string PrototypeDocument
        {
            get { return prototypeDocument; }
            set { prototypeDocument = value; this.IsDirty = true; }
        }

        /// <include file="doc\PropertyPages.uex" path="docs/doc[@for=&quot;GeneralPropertyPage.PrototypeType&quot;]/*"/>
        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeCaption")]
        [SRDescriptionAttribute("Optional. The tooltip for the prototype in the Toolbox. This value defaults to the <typeName> value specified.")]
        public String PrototypeCaption
        {
            get { return prototypeCaption; }
            set { prototypeCaption = value; this.IsDirty = true; }
        }

        /// <include file="doc\PropertyPages.uex" path="docs/doc[@for=&quot;GeneralPropertyPage.PrototypeType&quot;]/*"/>
        [SRCategoryAttribute(SR.Prototype)]
        [LocDisplayName("PrototypeBitmap")]
        [SRDescriptionAttribute("Optional. The filename of a 16x16 pixel bitmap to display as the Toolbox icon. This value defaults to the standard SystemDefinition icon for application and logical server prototypes, and to the standard EndpointDefinition icon for endpoint prototypes.")]
        public String PrototypeBitmap
        {
            get { return prototypeBitmap; }
            set { prototypeBitmap = value; this.IsDirty = true; }
        }

        protected override void BindProperties()
        {
            if (this.ProjectMgr == null)
            {
                Debug.Assert(false);
                return;
            }

            string prototypeType = this.ProjectMgr.GetProjectProperty("PrototypeType", false);
            if (prototypeType != null && prototypeType.Length > 0)
                try { this.prototypeType = (PrototypeType)Enum.Parse(typeof(PrototypeType), prototypeType); }
                catch { } //Should only fail if project file is corrupt

            this.prototypeCaption = this.ProjectMgr.GetProjectProperty("PrototypeCaption", false);
            this.prototypeBitmap = this.ProjectMgr.GetProjectProperty("PrototypeBitmap", false);
            this.prototypeTypeName = this.ProjectMgr.GetProjectProperty("PrototypeTypeName", false);

            String prototypeLayerEnumName = this.ProjectMgr.GetProjectProperty("PrototypeLayer", false);
            if (prototypeLayerEnumName != null && prototypeLayerEnumName.Length > 0)
                try { this.prototypeLayer = (PrototypeLayer)Enum.Parse(typeof(PrototypeLayer), prototypeLayerEnumName); }
                catch { } //Should only fail if project file is corrupt

            this.prototypeDocument = this.ProjectMgr.GetProjectProperty("PrototypeDocument", false);
            this.prototypeName = this.ProjectMgr.GetProjectProperty("PrototypeName", false);
            this.prototypeOutput = this.ProjectMgr.GetProjectProperty("PrototypeOutput", false);
        }

        protected override int ApplyChanges()
        {
            if (this.ProjectMgr == null)
            {
                Debug.Assert(false);
                return VSConstants.E_INVALIDARG;
            }

            this.ProjectMgr.SetProjectProperty("PrototypeType", this.prototypeType.ToString());
            this.ProjectMgr.SetProjectProperty("PrototypeCaption", this.prototypeCaption);
            this.ProjectMgr.SetProjectProperty("PrototypeBitmap", this.prototypeBitmap);
            this.ProjectMgr.SetProjectProperty("PrototypeTypeName", this.prototypeTypeName);
            this.ProjectMgr.SetProjectProperty("PrototypeLayer", this.prototypeLayer.ToString());
            this.ProjectMgr.SetProjectProperty("PrototypeDocument", this.prototypeDocument);
            this.ProjectMgr.SetProjectProperty("PrototypeName", this.prototypeName);

            this.prototypeOutput = this.ProjectMgr.GetProjectProperty("PrototypeOutput", false);

            return VSConstants.S_OK;
        }
    }
}
