﻿
/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

namespace Eastbanc.VSIP.Sdm.VSProject
{
    using System;
    using System.Reflection;
    using System.Globalization;
    using System.Resources;
    using System.Text;
    using System.Threading;
    using System.ComponentModel;
    using System.Security.Permissions;

    [AttributeUsage(AttributeTargets.All)]
    internal sealed class SRDescriptionAttribute : DescriptionAttribute
    {

        private bool replaced = false;

        public SRDescriptionAttribute(string description)
            : base(description)
        {
        }

        public override string Description
        {
            get
            {
                if (!replaced)
                {
                    replaced = true;
                    DescriptionValue = SR.GetString(base.Description);
                }
                return base.Description;
            }
        }
    }

    [AttributeUsage(AttributeTargets.All)]
    internal sealed class SRCategoryAttribute : CategoryAttribute
    {

        public SRCategoryAttribute(string category)
            : base(category)
        {
        }

        protected override string GetLocalizedString(string value)
        {
            return SR.GetString(value);
        }
    }
    internal sealed class SR
    {
        internal const string Application = "Application";
        internal const string GeneralCaption = "GeneralCaption";
        internal const string AssemblyName = "AssemblyName";
        internal const string AssemblyNameDescription = "AssemblyNameDescription";
        internal const string OutputType = "OutputType";
        internal const string OutputTypeDescription = "OutputTypeDescription";
        internal const string DefaultNamespace = "DefaultNamespace";
        internal const string DefaultNamespaceDescription = "DefaultNamespaceDescription";
        internal const string StartupObject = "StartupObject";
        internal const string StartupObjectDescription = "StartupObjectDescription";
        internal const string ApplicationIcon = "ApplicationIcon";
        internal const string ApplicationIconDescription = "ApplicationIconDescription";
        internal const string Project = "Project";
        internal const string ProjectFile = "ProjectFile";
        internal const string ProjectFileDescription = "ProjectFileDescription";
        internal const string ProjectFolder = "ProjectFolder";
        internal const string ProjectFolderDescription = "ProjectFolderDescription";
        internal const string OutputFile = "OutputFile";
        internal const string OutputFileDescription = "OutputFileDescription";
        internal const string TargetPlatform = "TargetPlatform";
        internal const string TargetPlatformDescription = "TargetPlatformDescription";
        internal const string TargetPlatformLocation = "TargetPlatformLocation";
        internal const string TargetPlatformLocationDescription = "TargetPlatformLocationDescription";
        internal const string SdmSourceDocument = "SdmSourceDocument";
        internal const string SdmSourceDocumentDescription = "SdmSourceDocumentDescription";
        internal const string SdmTargetDocument = "SdmTargetDocument";
        internal const string SdmTargetDocumentDescription = "SdmTargetDocumentDescription";
        internal const string SdmDocument = "SdmDocument";
        internal const string Prototype = "Prototype";
        internal const string PrototypeCaption = "PrototypeCaption";
        internal const string SdmReferenceSearchPath = "SdmReferenceSearchPath";
        internal const string SdmReferenceSearchPathDescription = "SdmReferenceSearchPathDescription";
        internal const string SdmResources = "SdmResources";
        internal const string SdmNoLogo = "SdmNoLogo";
        internal const string SdmMaxComplexity = "SdmMaxComplexity";
        internal const string InstanceSpaceReport = "InstanceSpaceReport";
        internal const string ConfigurationReport = "ConfigurationReport";
        internal const string SdmWarningLevelAsErrors = "SdmWarningLevelAsErrors";
        internal const string SdmWarningLevelReported = "SdmWarningLevelReported";
        internal const string SdmErrorLimit = "SdmErrorLimit";
        internal const string GenerateClassses = "GenerateClassses";
        internal const string GenerateResources = "GenerateResources";
        internal const string GeneratedClassFile = "GeneratedClassFile";
        internal const string GeneratedResourceFile = "GeneratedResourceFile";
        internal const string GenerateSettingClasses = "GenerateSettingClasses";
        internal const string GenerateConstraintClasses = "GenerateConstraintClasses";
        internal const string GenerateFlowClasses = "GenerateFlowClasses";
        internal const string ResourceTextFile = "ResourceTextFile";
        internal const string SdmNoLogoDescription = "SdmNoLogoDescription";
        internal const string SdmMaxComplexityDescription = "SdmMaxComplexityDescription";
        internal const string InstanceSpaceReportDescription = "InstanceSpaceReportDescription";
        internal const string ConfigurationReportDescription = "ConfigurationReportDescription";
        internal const string SdmWarningLevelAsErrorsDescription = "SdmWarningLevelAsErrorsDescription";
        internal const string SdmWarningLevelReportedDescription = "SdmWarningLevelReportedDescription";
        internal const string SdmErrorLimitDescription = "SdmErrorLimitDescription";
        internal const string GenerateClasssesDescription = "GenerateClasssesDescription";
        internal const string GenerateResourcesDescription = "GenerateResourcesDescription";
        internal const string GeneratedClassFileDescription = "GeneratedClassFileDescription";
        internal const string GeneratedResourceFileDescription = "GeneratedResourceFileDescription";
        internal const string GenerateSettingClassesDescription = "GenerateSettingClassesDescription";
        internal const string GenerateConstraintClassesDescription = "GenerateConstraintClassesDescription";
        internal const string GenerateFlowClassesDescription = "GenerateFlowClassesDescription";
        internal const string ResourceTextFileDescription = "ResourceTextFileDescription";
        internal const string SdmResourceCaption = "SdmResourceCaption";
        internal const string SdmDocumentCaption = "SdmDocumentCaption";
        internal const string SdmTargetDocumentPath = "SdmTargetDocumentPath";
        internal const string SdmTargetDocumentPathDescription = "SdmTargetDocumentPathDescription";
        internal const string RegisterSdmOutput = "RegisterSdmOutput";
        internal const string RegisterSdmOutputDescription = "RegisterSdmOutputDescription";
        internal const string TargetRegistryRoot = "TargetRegistryRoot";
        internal const string TargetRegistryRootDescription = "TargetRegistryRootDescription";
        internal const string CopyToOutputDirectory = "CopyToOutputDirectory";
        internal const string CopyToOutputDirectoryDescription = "CopyToOutputDirectoryDescription";
                
        static SR loader = null;
        ResourceManager resources;

        private static Object s_InternalSyncObject;
        private static Object InternalSyncObject
        {
            get
            {
                if (s_InternalSyncObject == null)
                {
                    Object o = new Object();
                    Interlocked.CompareExchange(ref s_InternalSyncObject, o, null);
                }
                return s_InternalSyncObject;
            }
        }

        internal SR()
        {
            resources = Eastbanc.VSIP.Sdm.VSProject.Resources.ResourceManager;
        }

        private static SR GetLoader()
        {
            if (loader == null)
            {
                lock (InternalSyncObject)
                {
                    if (loader == null)
                    {
                        loader = new SR();
                    }
                }
            }

            return loader;
        }

        private static CultureInfo Culture
        {
            get { return null/*use ResourceManager default, CultureInfo.CurrentUICulture*/; }
        }

        public static ResourceManager Resources
        {
            get
            {
                return GetLoader().resources;
            }
        }

        public static string GetString(string name, params object[] args)
        {
            SR sys = GetLoader();
            if (sys == null)
                return null;
            string res = sys.resources.GetString(name, SR.Culture);

            if (args != null && args.Length > 0)
            {
                return String.Format(CultureInfo.CurrentCulture, res, args);
            }
            else
            {
                return res;
            }
        }

        public static string GetString(string name)
        {
            SR sys = GetLoader();
            if (sys == null)
                return null;
            return sys.resources.GetString(name, SR.Culture);
        }

        public static object GetObject(string name)
        {
            SR sys = GetLoader();
            if (sys == null)
                return null;
            return sys.resources.GetObject(name, SR.Culture);
        }
    }
}
