
/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using System.Text;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Package;
using Microsoft.Win32;
using EnvDTE;
using IOleServiceProvider = Microsoft.VisualStudio.OLE.Interop.IServiceProvider;
using Microsoft.VisualStudio;

using Eastbanc.VSIP.Sdm;

namespace Eastbanc.VSIP.Sdm.VSProject
{
    [DefaultRegistryRoot("Software\\Microsoft\\VisualStudio\\8.0")]
    [ProvideProjectFactory(typeof(SDMElementProjectFactory), SDMElementProject.ProjectTypeName, "Sdm Project Files (*.sdmproj);*.sdmproj", "sdmproj", "sdmproj", null, LanguageVsTemplate = SDMElementProject.ProjectTypeName)]//, TemplateGroupIDsVsTemplate = "SDM-Custom-v1", ShowOnlySpecifiedTemplatesVsTemplate = true, TemplateIDsVsTemplate = "AssemblyInfo,Bitmap,BlankDatabase,ClassDiagram,CodeFile,Cursor,DataSet,HTMLPage,Icon,Interface,Resource,Stylesheet,Text,XMLFile,XSDSchema,XSLTFile,Module", NewProjectRequireNewFolderVsTemplate=false)]
    //[ProvideProjectItem(typeof(SDMElementProjectFactory), "My Items", @"C:\Documents and Settings\popove\My Documents\Visual Studio 2005\Projects\Eastbanc Projects\VSIP\SDM\SDMCustomProject\Templates\Templates\ProjectItems", 500)]
    [ProvideObject(typeof(GeneralPropertyPage))]
    [ProvideObject(typeof(PrototypePropertyPage))]
    [ProvideObject(typeof(SdmDocumentPropertyPage))]
    [ProvideObject(typeof(ConfigurationPropertyPage))]
    [ProvideObject(typeof(SdmResourcePropertyPage))]
    [Microsoft.VisualStudio.Shell.PackageRegistration(RegisterUsing = RegistrationMethod.Assembly, UseManagedResourcesOnly = true)]
    [Guid("e815d8ad-49bf-427d-99c8-009db3e56ab9")]
    public class SDMPackage : ProjectPackage
    {
        protected override void Initialize()
        {
            base.Initialize();
            this.RegisterProjectFactory(new SDMElementProjectFactory(this));
        }
    }

    [GuidAttribute("952a0f31-c8d1-4365-9936-107f60fa614d")]
    public class SDMElementProjectFactory : Microsoft.VisualStudio.Package.ProjectFactory
    {
        public SDMElementProjectFactory(SDMPackage package)
            : base(package)
        {
        }

        protected override Microsoft.VisualStudio.Package.ProjectNode CreateProject()
        {
            SDMElementProject project = new SDMElementProject(this.Package as SDMPackage);
            project.SetSite((IOleServiceProvider)((IServiceProvider)this.Package).GetService(typeof(IOleServiceProvider)));
            return project;
        }
    }

    [Guid("919fc76c-2c18-4df3-818a-e2a4da45d915")]
    public class SDMElementProject : Microsoft.VisualStudio.Package.ProjectNode
    {
        public const String ProjectTypeName = "SDMComponent";

        private Guid GUID_MruPage = new Guid("{19B97F03-9594-4c1c-BE28-25FF030113B3}");
        private SDMPackage package;
        private VSLangProj.VSProject vsProject = null;
        private const String SDM_MODELS_REG_PATH = @"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\VisualStudio\8.0\EnterpriseTools\Sdm\SearchFolders";
        

        public SDMElementProject(SDMPackage pkg)
        {
            this.package = pkg;
            this.NodeProperties = new ProjectNodeProperties(this);
            this.OleServiceProvider.AddService(typeof(VSLangProj.VSProject), this.VSProject, false);
            this.CanFileNodesHaveChilds = true;
            this.SupportsProjectDesigner = true;

            InitializeCATIDs();
        }

        private void InitializeCATIDs()
        {
            this.AddCATIDMapping(typeof(FolderNodeProperties), new Guid("A3273B8E-FDF8-4ea8-901B-0D66889F645F"));
            this.AddCATIDMapping(typeof(ReferenceNodeProperties), typeof(ReferenceNodeProperties).GUID);
            this.AddCATIDMapping(typeof(SdmFileNodeProperties), typeof(SdmFileNodeProperties).GUID);
            this.AddCATIDMapping(typeof(FileNodeProperties), typeof(SdmFileNodeProperties).GUID);
        }
        
        public override Guid ProjectGuid
        {
            get
            {
                return typeof(SDMElementProjectFactory).GUID;
            }
        }
        
        public override string ProjectType
        {
            get
            {
                return ProjectTypeName;
            }
        }

        /// <summary>
        /// Get the VSProject corresponding to this project
        /// </summary>
        protected internal VSLangProj.VSProject VSProject
        {
            get
            {
                if (vsProject == null)
                    vsProject = new Microsoft.VisualStudio.Package.Automation.OAVSProject(this);
                return vsProject;
            }
        }

        
        /// <summary>
        /// Overriding to provide project general property page
        /// </summary>
        /// <returns></returns>
        protected override Guid[] GetConfigurationIndependentPropertyPages()
        {
            System.Collections.Generic.List<Guid> guids = new System.Collections.Generic.List<Guid>(5);
                      
            guids.Add(typeof(GeneralPropertyPage).GUID);

            switch (this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.OutputType))
            {
                case "SdmDocument":
                    guids.Add(typeof(SdmDocumentPropertyPage).GUID); 
                    break;
                case "SdmPrototype":
                    guids.Add(typeof(PrototypePropertyPage).GUID);
                    break;
                case "SdmResource":
                    guids.Add(typeof(SdmDocumentPropertyPage).GUID);
                    guids.Add(typeof(SdmResourcePropertyPage).GUID);
                    break;
            }
            
            return guids.ToArray();
        }

        /// <summary>
        /// Returns the configuration dependent property pages.
        /// Specify here a property page. By returning no property page the configuartion dependent properties will be neglected.
        /// Overriding, but current implementation does nothing
        /// To provide configuration specific page project property page, this should return an array bigger then 0
        /// (you can make it do the same as GetPropertyPageGuids() to see its impact)
        /// </summary>
        /// <param name="config"></param>
        /// <returns></returns>
        protected override Guid[] GetConfigurationDependentPropertyPages()
        {
            // TODO: Remove this if fixed.
            // There is a bug in the \vbdesigner\designer\ApplicationDesigner\ApplicationDesignerView.vb  
            // and we cannot have configuration independent property pages. 
            // See the method GetPropertyPages in that class.
            if (this.SupportsProjectDesigner)
            {
                return this.GetConfigurationIndependentPropertyPages();
            }

            Guid[] result = new Guid[1];
            result[0] = typeof(ConfigurationPropertyPage).GUID;
            return result;
        }
        
        /// <summary>
        /// Overriding to provide customization of files on add files.
        /// This will replace tokens in the file with actual value (namespace, class name,...)
        /// </summary>
        /// <param name="source">Full path to template file</param>
        /// <param name="target">Full path to destination file</param>
        public override void AddFileFromTemplate(string source, string target)
        {
            if (!System.IO.File.Exists(source))
                throw new FileNotFoundException(String.Format("Template file not found: {0}", source));
                        
            // The class name is based on the new file name
            string className = Path.GetFileNameWithoutExtension(target);
            string targetName = this.ProjectMgr.GetProjectProperty("Name", false);
            string projectName = this.ProjectMgr.GetProjectProperty("Name", false);
            string namespce = targetName;
            string sdmElementName = targetName;
            string sdmdocumentname = targetName;

            this.FileTemplateProcessor.AddReplace("%classname%", className);
            this.FileTemplateProcessor.AddReplace("%namespace%", namespce);
            this.FileTemplateProcessor.AddReplace("%targetname%", targetName);
            this.FileTemplateProcessor.AddReplace("%safeprojectname%", projectName);
            this.FileTemplateProcessor.AddReplace("%sdmElementName%", sdmElementName);
            this.FileTemplateProcessor.AddReplace("%sdmdocumentname%", sdmdocumentname);
                        
            try
            {
                //tokenReplacer.UntokenMiscFile(source, target);
                this.FileTemplateProcessor.UntokenFile(source, target);
            }
            catch (Exception e)
            {
                throw new FileLoadException("Failed to add template file to project", target, e);
            }
        }

        public override FileNode CreateFileNode(ProjectElement item)
        {
            if (item == null)
            {
                throw new ArgumentNullException("item");
            }

            string include = item.GetMetadata(ProjectFileConstants.Include);
            SdmFileNode newNode = new SdmFileNode(this, item);
            
            if (IsCodeFile(include))
            {
                newNode.OleServiceProvider.AddService(typeof(VSLangProj.VSProject), this.VSProject, false);
            }
            
            return newNode;

            //return base.CreateFileNode(item);
        }

        public override bool IsCodeFile(string strFileName)
        {
            if (strFileName.EndsWith(".cs"))
                return true;

            if (strFileName.EndsWith(".sdmx"))
                return true;

            return base.IsCodeFile(strFileName);
        }
        
        protected override ReferenceContainerNode CreateReferenceContanerNode()
        {
            return new SdmDocumentReferenceContainerNode(this);
        }

        protected override void Reload()
        {
            base.Reload();

            ProcessSdmDocumentFiles();
        }

        private void ProcessSdmDocumentFiles()
        {
            
        }

        /// <summary>
        /// This overrides the base class method to show the VS 2005 style Add reference dialog. The ProjectNode implementation
        /// shows the VS 2003 style Add Reference dialog.
        /// </summary>
        /// <returns>S_OK if succeeded. Failure other wise</returns>
        public override int AddProjectReference()
        {
            IVsComponentSelectorDlg2 componentDialog;
            Guid guidEmpty = Guid.Empty;
            VSCOMPONENTSELECTORTABINIT[] tabInit = new VSCOMPONENTSELECTORTABINIT[5];
            string strBrowseLocations = (String)Registry.GetValue(SDM_MODELS_REG_PATH, "SdmFolder1", Path.GetDirectoryName(this.BaseURI.Uri.LocalPath));

            //Add the .NET page
            tabInit[0].dwSize = (uint)Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[0].varTabInitInfo = 0;
            tabInit[0].guidTab = VSConstants.GUID_COMPlusPage;

            //Add the Project page
            tabInit[1].dwSize = (uint)Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            // Tell the Add Reference dialog to call hierarchies GetProperty with the following
            // propID to enablefiltering out ourself from the Project to Project reference
            tabInit[1].varTabInitInfo = (int)__VSHPROPID.VSHPROPID_ShowProjInSolutionPage;
            tabInit[1].guidTab = VSConstants.GUID_SolutionPage;

            // Add the Browse page			
            tabInit[2].dwSize = (uint)Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[2].guidTab = NativeMethods.GUID_BrowseFilePage;
            tabInit[2].varTabInitInfo = 0;

            //// Add the Recent page			
            tabInit[3].dwSize = (uint)Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[3].guidTab = GUID_MruPage;
            tabInit[3].varTabInitInfo = 0;

            uint pX = 0, pY = 0;


            componentDialog = this.GetService(typeof(SVsComponentSelectorDlg)) as IVsComponentSelectorDlg2;
            try
            {
                // call the container to open the add reference dialog.
                if (componentDialog != null)
                {
                    // Let the project know not to show itself in the Add Project Reference Dialog page
                    this.ShowProjectInSolutionPage = false;

                    // call the container to open the add reference dialog.
                    ErrorHandler.ThrowOnFailure(componentDialog.ComponentSelectorDlg2(
                        (System.UInt32)(__VSCOMPSELFLAGS.VSCOMSEL_MultiSelectMode | __VSCOMPSELFLAGS.VSCOMSEL_IgnoreMachineName),
                        (IVsComponentUser)this,
                        0,
                        null,
                        "Add References",//Microsoft.VisualStudio.Package.SR.GetString(Microsoft.VisualStudio.Package.SR.AddReferenceDialogTitle),   // Title
                        "VS.AddReference",						  // Help topic
                        ref pX,
                        ref pY,
                        (uint)tabInit.Length,
                        tabInit,
                        ref guidEmpty,
                        "*.sdmDocument;*.dll",
                        ref strBrowseLocations));
                }
            }
            catch (COMException e)
            {
                Trace.WriteLine("Exception : " + e.Message);
                return e.ErrorCode;
            }
            finally
            {
                // Let the project know it can show itself in the Add Project Reference Dialog page
                this.ShowProjectInSolutionPage = true;
            }
            return VSConstants.S_OK;
        }
    }
}