using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Microsoft.SystemDefinitionModel.Tools;

namespace Eastbanc.VSIP.Sdm.VSProject
{
    [ComVisible(true), Guid("3c693da2-5bca-49b3-bd95-ffe0a39dd999")]
    public class SdmDocumentPropertyPage : Microsoft.VisualStudio.Package.SettingsPage
    {
        private String sdmTargetDocumentPath;
        private String sdmTargetDocumentFile;
        private String sdmReferenceSearchPath;
        private String sdmSourceDocumentFile;
        private bool sdmNoLogo;
        private int sdmMaxComplexity;
        private String instanceSpaceReport;
        private String configurationReport;
        private WarningLevel sdmWarningLevelAsErrors;
        private WarningLevel sdmWarningLevelReported;
        private int sdmErrorLimit;

    
        public SdmDocumentPropertyPage()
        {
            this.Name = Resources.SdmDocumentCaption;
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmTargetDocument)]
        [SRDescriptionAttribute(SR.SdmTargetDocumentDescription)]
        public String SdmTargetDocumentFile
        {
            get { return sdmTargetDocumentFile; }
            set { sdmTargetDocumentFile = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmSourceDocument)]
        [SRDescriptionAttribute(SR.SdmSourceDocumentDescription)]
        public String SdmSourceDocumentFile
        {
            get { return sdmSourceDocumentFile; }
            set { sdmSourceDocumentFile = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmReferenceSearchPath)]
        [SRDescriptionAttribute(SR.SdmReferenceSearchPathDescription)]
        public String SdmReferenceSearchPath
        {
            get { return sdmReferenceSearchPath; }
            set { sdmReferenceSearchPath = value; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.ConfigurationReport)]
        [SRDescriptionAttribute(SR.ConfigurationReportDescription)]
        public String ConfigurationReport
        {
            get { return configurationReport; }
            set { configurationReport = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.InstanceSpaceReport)]
        [SRDescriptionAttribute(SR.InstanceSpaceReportDescription)]
        public String InstanceSpaceReport
        {
            get { return instanceSpaceReport; }
            set { instanceSpaceReport = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmWarningLevelAsErrors)]
        [SRDescriptionAttribute(SR.SdmWarningLevelAsErrorsDescription)]
        public WarningLevel SdmWarningLevelAsErrors
        {
            get { return sdmWarningLevelAsErrors; }
            set { sdmWarningLevelAsErrors = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmWarningLevelReported)]
        [SRDescriptionAttribute(SR.SdmWarningLevelReportedDescription)]
        public WarningLevel SdmWarningLevelReported
        {
            get { return sdmWarningLevelReported; }
            set { sdmWarningLevelReported = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmErrorLimit)]
        [SRDescriptionAttribute(SR.SdmErrorLimitDescription)]
        public int SdmErrorLimit
        {
            get { return sdmErrorLimit; }
            set { sdmErrorLimit = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmMaxComplexity)]
        [SRDescriptionAttribute(SR.SdmMaxComplexityDescription)]
        public int SdmMaxComplexity
        {
            get { return sdmMaxComplexity; }
            set { sdmMaxComplexity = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmNoLogo)]
        [SRDescriptionAttribute(SR.SdmNoLogoDescription)]
        public bool SdmNoLogo
        {
            get { return sdmNoLogo; }
            set { sdmNoLogo = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmDocument)]
        [LocDisplayName(SR.SdmTargetDocumentPath)]
        [SRDescriptionAttribute(SR.SdmTargetDocumentPathDescription)]
        public String SdmTargetDocumentPath
        {
            get { return sdmTargetDocumentPath; }
        }

        public override string GetClassName()
        {
            return this.GetType().FullName;
        }


        protected override void BindProperties()
        {
            if (this.ProjectMgr == null)
            {
                Debug.Assert(false);
                return;
            }

            this.sdmSourceDocumentFile = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmSourceFile, false);
            this.sdmTargetDocumentFile = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmTargetFile, false);
            this.sdmNoLogo = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmNoLogo, false), false);
            this.sdmMaxComplexity = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmMaxComplexity, false), 10);
            this.instanceSpaceReport = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.InstanceSpaceReport, false);
            this.configurationReport = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.ConfigurationReport, false);
            this.sdmWarningLevelAsErrors = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmWarningLevelAsErrors, false), WarningLevel.Warning3);
            this.sdmWarningLevelReported = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmWarningLevelReported, false), WarningLevel.Warning1);
            this.sdmErrorLimit = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmErrorLimit, false), 100);
            this.sdmReferenceSearchPath = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmReferenceSearchPath, false);
            this.sdmTargetDocumentPath = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmTargetDocumentPath, false);
        }

        protected override int ApplyChanges()
        {
            if (this.ProjectMgr == null)
            {
                Debug.Assert(false);
                return VSConstants.E_INVALIDARG;
            }

            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmSourceFile, this.sdmSourceDocumentFile);
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmTargetFile, this.sdmTargetDocumentFile);

            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmNoLogo, this.sdmNoLogo.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmMaxComplexity, this.sdmMaxComplexity.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.InstanceSpaceReport, this.instanceSpaceReport);
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.ConfigurationReport, this.configurationReport);
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmWarningLevelAsErrors, Utilities.ConvertToProjectProperty(this.sdmWarningLevelAsErrors));
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmWarningLevelReported, Utilities.ConvertToProjectProperty(this.sdmWarningLevelReported));
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmErrorLimit, this.sdmErrorLimit.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.SdmReferenceSearchPath, this.sdmReferenceSearchPath);

            this.sdmTargetDocumentPath = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.SdmTargetDocumentPath, false);

            IsDirty = false;

            return VSConstants.S_OK;
        }
    }
}
