using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.UI;
using Microsoft.VisualStudio.Package;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Globalization;
using System.Diagnostics;

namespace Eastbanc.VSIP.Sdm.VSProject
{
    public class CopyToOutputDirectoryConverter : EnumConverter
    {
        public CopyToOutputDirectoryConverter()
			: base(typeof(CopyToOutputDirectory))
		{ 
		
		}

        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string)) return true;

            return base.CanConvertFrom(context, sourceType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            string str = value as string;

            if (str != null)
            {
                if (str == Resources.Always) return CopyToOutputDirectory.Always;

                if (str == Resources.Never) return CopyToOutputDirectory.Never;

                if (str == Resources.Newer) return CopyToOutputDirectory.Newer;
            }

            return base.ConvertFrom(context, culture, value);
        }
        

        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                string result = null;

                // In some cases if multiple nodes are selected the windows form engine
                // calls us with a null value if the selected node's property values are not equal
                // Example of windows form engine passing us null: File set to Compile, Another file set to None, bot nodes are selected, and the build action combo is clicked.
                if (value != null)
                {
                    result = SR.GetString(((CopyToOutputDirectory)value).ToString(), culture);
                }
                else
                {
                    result = SR.GetString(CopyToOutputDirectory.Never.ToString(), culture);
                }

                if (result != null) return result;
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }

        public override bool GetStandardValuesSupported(System.ComponentModel.ITypeDescriptorContext context)
        {
            return true;
        }

        public override System.ComponentModel.TypeConverter.StandardValuesCollection GetStandardValues(System.ComponentModel.ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new CopyToOutputDirectory[] { CopyToOutputDirectory.Always, CopyToOutputDirectory.Never, CopyToOutputDirectory.Newer });
        }
    }

    [PropertyPageTypeConverterAttribute(typeof(CopyToOutputDirectoryConverter))]
    public enum CopyToOutputDirectory
    {
        Always,
        Newer,
        Never
    }

    [CLSCompliant(false), ComVisible(true)]
    public class SdmFileNodeProperties: FileNodeProperties
    {
        #region fields
        protected CopyToOutputDirectory _copyToOutputDirectory;
        #endregion

        #region properties
        [SRCategoryAttribute(Microsoft.VisualStudio.Package.SR.Advanced)]
        [LocDisplayName(SR.CopyToOutputDirectory)]
        [SRDescriptionAttribute(SR.CopyToOutputDirectoryDescription)]
        public virtual CopyToOutputDirectory CopyToOutputDirectory
        {
            get
            {
                Debug.Assert(this.Node != null, "The associated hierarchy node has not been initialized");
                string value = this.GetProperty(SdmProjectFileConstants.CopyToOutputDirectory, null);
                
                if (value == null || value.Length == 0)
                    return CopyToOutputDirectory.Never;

                return (CopyToOutputDirectory)Enum.Parse(typeof(CopyToOutputDirectory), value);
            }
            set
            {
                Debug.Assert(this.Node != null, "The associated hierarchy node has not been initialized");

                if(value == CopyToOutputDirectory.Never)
                    this.SetProperty(SdmProjectFileConstants.CopyToOutputDirectory, null);
                else
                    this.SetProperty(SdmProjectFileConstants.CopyToOutputDirectory, value.ToString());
            }
        }
        #endregion

        #region ctors
		public SdmFileNodeProperties()
		{
		}

        public SdmFileNodeProperties(HierarchyNode node)
			: base(node)
		{
		}
		#endregion

    }
}
