using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Eastbanc.VSIP.Sdm.VSProject
{
    [ComVisible(true), Guid("AA46F72C-8CDE-45da-AC66-94782288C51A")]
    public class SdmResourcePropertyPage : Microsoft.VisualStudio.Package.SettingsPage
    {
        private bool generateClassses;
        private bool generateResources;
        private String generatedClassFile;
        private String generatedResourceFile;
        private bool generateSettingClasses;
        private bool generateConstraintClasses;
        private bool generateFlowClasses;
        private String resourceTextFile;

        public SdmResourcePropertyPage()
        {
            this.Name = Resources.SdmResources;
        }

        protected override void BindProperties()
        {
            if (this.ProjectMgr == null)
            {
                Debug.Assert(false);
                return;
            }

            this.generateClassses = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GenerateClassses, false), true);
            this.generateResources = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GenerateResources, false), true);
            this.generatedClassFile = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GeneratedClassFile, false);
            this.generatedResourceFile = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GeneratedResourceFile, false);
            this.generateSettingClasses = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GenerateSettingClasses, false), true);
            this.generateConstraintClasses = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GenerateConstraintClasses, false), true);
            this.generateFlowClasses = Utilities.ConvertFromProjectProperty(this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.GenerateFlowClasses, false), true);
            this.resourceTextFile = this.ProjectMgr.GetProjectProperty(SdmProjectFileConstants.ResourceTextFile, false);

        }

        protected override int ApplyChanges()
        {
            if (this.ProjectMgr == null)
            {
                Debug.Assert(false);
                return VSConstants.E_INVALIDARG;
            }

            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GenerateClassses, this.generateClassses.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GenerateResources, this.generateResources.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GeneratedClassFile, this.generatedClassFile);
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GeneratedResourceFile, this.generatedResourceFile);
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GenerateSettingClasses, this.generateSettingClasses.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GenerateConstraintClasses, this.generateConstraintClasses.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.GenerateFlowClasses, this.generateFlowClasses.ToString());
            this.ProjectMgr.SetProjectProperty(SdmProjectFileConstants.ResourceTextFile, this.resourceTextFile);

            IsDirty = false;

            return VSConstants.S_OK;
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GenerateClassses)]
        [SRDescriptionAttribute(SR.GenerateClasssesDescription)]
        public bool GenerateClassses
        {
            get { return generateClassses; }
            set { generateClassses = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GenerateResources)]
        [SRDescriptionAttribute(SR.GenerateResourcesDescription)]
        public bool GenerateResources
        {
            get { return generateResources; }
            set { generateResources = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GeneratedClassFile)]
        [SRDescriptionAttribute(SR.GeneratedClassFileDescription)]
        public String GeneratedClassFile
        {
            get { return generatedClassFile; }
            set { generatedClassFile = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GeneratedResourceFile)]
        [SRDescriptionAttribute(SR.GeneratedResourceFileDescription)]
        public String GeneratedResourceFile
        {
            get { return generatedResourceFile; }
            set { generatedResourceFile = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GenerateSettingClasses)]
        [SRDescriptionAttribute(SR.GenerateSettingClassesDescription)]
        public bool GenerateSettingClasses
        {
            get { return generateSettingClasses; }
            set { generateSettingClasses = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GenerateConstraintClasses)]
        [SRDescriptionAttribute(SR.GenerateConstraintClassesDescription)]
        public bool GenerateConstraintClasses
        {
            get { return generateConstraintClasses; }
            set { generateConstraintClasses = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.GenerateFlowClasses)]
        [SRDescriptionAttribute(SR.GenerateFlowClassesDescription)]
        public bool GenerateFlowClasses
        {
            get { return generateFlowClasses; }
            set { generateFlowClasses = value; IsDirty = true; }
        }

        [SRCategoryAttribute(SR.SdmResourceCaption)]
        [LocDisplayName(SR.ResourceTextFile)]
        [SRDescriptionAttribute(SR.ResourceTextFileDescription)]
        public String ResourceTextFile
        {
            get { return resourceTextFile; }
            set { resourceTextFile = value; IsDirty = true; }
        }
    }
}
