using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TemplateWizard;
using System.IO;
using Microsoft.Win32;
using EnvDTE;
using Microsoft.VisualStudio.Package;

namespace Eastbanc.VSIP.Sdm.Wizards
{
    public class SdmSystemProjectWizardExtensions : WizardExtensionsBaseImplementation
    {
        public const String SdmWizardPrototypeDataKey = "$PrototypeDescription$";
        public const String SdmProjectPackageFactoryGuid = "{952a0f31-c8d1-4365-9936-107f60fa614d}";

        protected const String sdmApplicationDefaultSystemTemplate = @"SDM App.zip/ApplicationSystem.vstemplate";
        protected const String sdmPrototypeDefaultTemplate = @"SDM Prototype.zip/Prototype.vstemplate";
        protected WizardData _sdmWizardData = null;
        
        private     String _sdmPackageProjectTemplatePath = null;
        private     String _sdmPackageProjectItemTemplatePath = null;
       
                
        public override void ProjectFinishedGenerating(EnvDTE.Project project)
        {
            base.ProjectFinishedGenerating(project);

            try
            {
                if (_sdmWizardData == null)
                    return;
                                
                String projectFolder = Path.GetDirectoryName(project.FullName);
                String currentProjectTemplateDir = Path.GetDirectoryName(_currentTemplateFilePath);
                String sdmPackageTemplateRootDir = Path.GetDirectoryName(currentProjectTemplateDir);
                sdmPackageTemplateRootDir = Directory.GetParent(currentProjectTemplateDir).FullName;
                String defaultPrototypeProjectTemplatePath = Path.Combine(sdmPackageTemplateRootDir, sdmPrototypeDefaultTemplate);
                String sdmCurrentDocumentName = _sdmWizardData.SdmDocumentName;
                              
                foreach (Prototype_t pr in _sdmWizardData.Prototypes)
                {
                    String projectTemplate = defaultPrototypeProjectTemplatePath;
                    String projectName = project.Name + "Prototype";

                    if(pr.TemplateData != null)
                    {
                        projectName = pr.TypeName;

                        if (!String.IsNullOrEmpty(pr.TemplateData.ProjectName))
                            projectName = pr.TemplateData.ProjectName;
                        
                        if(pr.TemplateData.Any != null && !String.IsNullOrEmpty(pr.TemplateData.Any.Value))
                            projectTemplate = Path.Combine(currentProjectTemplateDir, sdmPrototypeDefaultTemplate);
                    }

                    EnvDTE.Project prototypeProject = _application.Solution.AddFromTemplate(projectTemplate, projectFolder, projectName, false);
                    prototypeProject = GetSolutionProjectByName(projectName);
                    ProjectNode newProject = prototypeProject.Object as ProjectNode;

                    newProject.SetProjectProperty(SdmProjectConstants.PrototypeType, pr.Type.ToString());
                    newProject.SetProjectProperty(SdmProjectConstants.PrototypeLayer, pr.Layer.ToString());
                    newProject.SetProjectProperty(SdmProjectConstants.PrototypeTypeName, pr.TypeName);
                    newProject.SetProjectProperty(SdmProjectConstants.PrototypeDocument, sdmCurrentDocumentName);

                    if (!String.IsNullOrEmpty(pr.Caption))
                        newProject.SetProjectProperty(SdmProjectConstants.PrototypeCaption, pr.Caption);

                    if (!String.IsNullOrEmpty(pr.Bitmap))
                        newProject.SetProjectProperty(SdmProjectConstants.PrototypeBitmap, pr.Bitmap);

                    if (!String.IsNullOrEmpty(pr.Name))
                        newProject.SetProjectProperty(SdmProjectConstants.PrototypeName, pr.Name);

                    newProject.SetProjectProperty(SdmProjectConstants.PrototypePosition, "");
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.Write(ex);
            }
        }

        protected override void  RunStarted()
        {
 
            if (!_replacementsDictionary.ContainsKey(SdmWizardPrototypeDataKey))
                return;

            try
            {
                String buffer = _replacementsDictionary[SdmWizardPrototypeDataKey];

                _processor.UntokenBuffer(ref buffer);

                String projectName = _replacementsDictionary["$safeprojectname$"];
                _sdmWizardData = new WizardData(projectName, projectName, SdmProjectWizardDataSerializationHelper.Deserialize(buffer));
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.Write(ex);
            }
        }

        protected String SdmProjectPackageRegistryPath
        {
            get
            {
                return Path.Combine(base.ProjectsRegistryRoot, SdmProjectPackageFactoryGuid);
            }
        }

        public String SdmProjectPackageProjectTemplatesDir
        {
            get
            {
                if (_sdmPackageProjectTemplatePath == null)
                {
                    _sdmPackageProjectTemplatePath = (string)Registry.GetValue(SdmProjectPackageRegistryPath, "ProjectTemplatesDir", "");
                }

                return _sdmPackageProjectTemplatePath;
            }
        }
    }
}
