using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TemplateWizard;
using System.IO;
using Microsoft.Win32;
using Microsoft.VisualStudio.Package;

namespace Eastbanc.VSIP.Sdm.Wizards
{
    public class WizardExtensionsBaseImplementation : Microsoft.VisualStudio.TemplateWizard.IWizard
    {
        public const String VSProjectsRigistryPath = "Projects";

        protected EnvDTE.DTE _application;
        protected TokenProcessor _processor;
        protected String _solutionFolderPath;
        protected WizardRunKind _runKind;
        protected object[] _customParams;
        protected String _currentTemplateFilePath;
        protected Dictionary<string, string> _replacementsDictionary;
        protected String _registryRoot;
        private   ProjectNode _activeProject;
        
        #region IWizard Members

        public virtual void BeforeOpeningFile(EnvDTE.ProjectItem projectItem)
        {
        }

        public virtual void ProjectFinishedGenerating(EnvDTE.Project project)
        {
            _activeProject = project.Object as ProjectNode;
        }

        public virtual void ProjectItemFinishedGenerating(EnvDTE.ProjectItem projectItem)
        {
        }

        public virtual void RunFinished()
        {
           
        }

        public void RunStarted(object automationObject, Dictionary<string, string> replacementsDictionary, Microsoft.VisualStudio.TemplateWizard.WizardRunKind runKind, object[] customParams)
        {
            _application = (EnvDTE.DTE)automationObject;
            _runKind = runKind;
            _customParams = customParams;
            _replacementsDictionary = replacementsDictionary;

            try
            {
                _registryRoot = Path.Combine(Registry.LocalMachine.Name, _application.RegistryRoot);

                if(_customParams != null && _customParams.Length != 0)
                    _currentTemplateFilePath = customParams[0] as string;

                _processor = new TokenProcessor();
                //_processor.TheDTE = _application;
                //_processor.ProjectNamespace = "";
                //_processor.ProjectName = replacementsDictionary["$projectname$"];

                foreach (string s in replacementsDictionary.Keys)
                {
                    _processor.AddReplace(s, replacementsDictionary[s]);
                }

                RunStarted();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.Write(ex);
            }
        }

        public virtual bool ShouldAddProjectItem(string filePath)
        {
            return true;
        }

        #endregion

        protected virtual void RunStarted()
        {
            
        }

        public String ProjectsRegistryRoot
        {
            get { return Path.Combine(_registryRoot, VSProjectsRigistryPath); }
        }

        protected ProjectNode ActiveProject
        {
            get
            {
                return _activeProject;
            }
        }

        protected EnvDTE.Project GetSolutionProjectByName(String name)
        {
            foreach (EnvDTE.Project pr in _application.Solution.Projects)
            {
                if (String.Compare(pr.Name, name, true) == 0)
                    return pr;
            }

            return null;
        }
    }
}
